/* ***************************************************************************
 *
 *	This program is free software; you can redistribute it and/or modify it
 * 	under the terms of the Shared Code License (SCL) version 1.0 as published
 *	by www.IntellectualHeaven.com. You should have received a copy of the SCL
 *	1.0 license, look for file license.txt, along with this software; if not,
 *	then please write to x_pankaj_x@intellectualheaven.com for an electronic
 *	copy of the same.
 *
 * File name:
 * 		Sf.cpp
 * 
 * Description:
 * 		This file implements functions which are common for both CUI and GUI
 *		build of SnifFire
 * 	
 * Revision:
 * --------------------------------------------------------------------------
 *		2005-04-24	xpankajx@intellectualheaven.com
 *		First version
 * --------------------------------------------------------------------------
 * 	Copyright (c) : Pankaj Garg
 * **************************************************************************/
 
// System-Includes!!!
#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <commdlg.h>


// Local-Includes!!!
#include "rawnetif.h"
#include "Sf.h"


/*
 * Global handle of the adapter which will be used to decide if adapter is
 * open or not
 */
HANDLE ghAdapter;

/*
 * Global event handle misc driver operations
 */
HANDLE ghEventAdapter;

/*
 * Global event handle for driver send operation
 */
HANDLE ghEventSend;

/*
 * Global event handle for driver receive operation
 */
HANDLE ghEventReceive;

/*
 * This decide how long to let the thread run
 */
bool gSnifferRun = true;

/*
 * Sniffer thread handle and id
 */
DWORD			gThreadId;
HANDLE			gThreadHandle;

/*
 * list to store deny IP list
 */
IPList gDenyIPList;

/*
 * Sniffer thread main function
 */
DWORD WINAPI SnifferThread(LPVOID pParam);

/*
 * Global view class pointer
 */
CSfView *gView;

/*
 * Global variable to indicate if the firewall is active or not
 */
bool gFirewallActivated = false;


/****************************************************************************
 *  FUNCTION NAME:  SetBitOn
 *
 *  DESCRIPTION:    Used to set a particular bit on in an integer
 *
 *  NOTES:          Bits are numbered from 0 - (sizeof(int)*8 - 1)
 *
 ***************************************************************************/
void
SetBitOn(	int *pValue,
			int BitNumber)
{
    *pValue |= (1 << BitNumber);
}


/****************************************************************************
 *  FUNCTION NAME:  ApplyFirewallOnPacket
 *
 *  DESCRIPTION:    Applies the firewall rules on packet and sets the action
 *					field of packet to action taken.
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
ApplyFirewallOnPacket(PacketData *inPacketData)
{
	/*
	 * We parse the packet data to find if it contains a TCP SYN request.
	 * Once we received a connect request, we can then decide if
	 * we need to send TCP RST or not, based on the Deny IP List
	 * loaded
	 */
	if (inPacketData->datagramType == IPV4_DATAGRAM)
	{
		if (inPacketData->tcp)
		{
			/*
			 * If we reach here, it means this is definitely a 
			 * TCP Packet
			 */
			bool foundInDenyList = false;

			for (int i = 0; i < gDenyIPList.size(); ++i)
			{
				if (inPacketData->ip->sourceIPAddr == gDenyIPList[i])
				{
					foundInDenyList = true;
					break;
				}
			}

			if (foundInDenyList)
			{
				/*
				 * We allow all packets from Denied machines other than
				 * TCP ACK, which is sent during connection handshake
				 */
				inPacketData->action = ACTION_ACCESS_GRANTED;
				
				if ((inPacketData->tcp->codeBits & 0x10) == 0x10)
				{
					/*
					 * This is an ACK message, which means the malicious IP
					 * has an open connection with us. RST the connection.
					 *
					 * Issue-Issue!!!
					 *
					 * The problem i faced with SYN is that even before i could
					 * send RST in reply to SYN, the underlying TCP Stack sent
					 * ACK + SYN and so my RST reply got ignored due to sequence
					 * numbering.
					 *
					 * Right now i am simply sending RST on every ACK received
					 * from the denied IP, this causes problems that even the
					 * connection originated from my machine to the denied
					 * machine are severed upon receiving an ACK from it.
					 *
					 * The right solution would be to send a reply a RST as soon
					 * as a SYN is received from the denied IP. If we need to
					 * beat the TCP stack implementation in our reponse time we
					 * should move this code to the driver itself such that the
					 * driver analyze the packet and sends RST back. We should
					 * also investigate if any other approach can be used
					 * instead of moving the code in the driver.
					 */
					SendTCPResetBack(inPacketData);
					inPacketData->action = ACTION_ACCESS_DENIED;
				}
			}
		}
	}
}



/****************************************************************************
 *  FUNCTION NAME:  SendTCPResetBack
 *
 *  DESCRIPTION:    If the given packet contains a TCP SYN request, then
 *					send TCP ACK + RST back
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
SendTCPResetBack(PacketData *inPacketData)
{
	/*
	 * Issue-Issue!!!
	 * We are always sending Ethernet_II packet in reply.
	 * Lets hope this doesn't cause any problem on 802.3 networks
	 */
	
	unsigned char 	TCPResetRes[64];
	int				dataIndex = 0;
	OVERLAPPED		overlapped;
	DWORD			bytesSent;

	memset(&overlapped, 0, sizeof(OVERLAPPED));

	/*
	 * Send RST to the denied machine
	 */
	Basic_TCP_Header tcpHeader;
	tcpHeader.sourcePort 	= inPacketData->tcp->destPort;
	tcpHeader.destPort 		= inPacketData->tcp->sourcePort;
	tcpHeader.seqNumber		= inPacketData->tcp->ackNumber;
	tcpHeader.ackNumber		= inPacketData->tcp->seqNumber;
	tcpHeader.hLen			= 0x50;		// we only use 4 high order bits
	tcpHeader.codeBits		= 0x04;		// RST bit set
	tcpHeader.window		= 0;
	tcpHeader.checksum		= 0;
	tcpHeader.urgentPointer	= 0;
	
	Basic_IP_Header	ipHeader;
	ipHeader.verAndHLen		= 0x45;		// Ver = 4, HLEN = 5 (20 bytes)
	ipHeader.tos			= 0;
	ipHeader.totalLength	= 0x2800;	// 20 bytes IP header and 20 TCP
	ipHeader.identification	= ~inPacketData->ip->identification;
	ipHeader.flagsAndOffset	= 0;
	ipHeader.ttl			= 64;
	ipHeader.proto			= TCP_PROTOCOL_NUM;
	ipHeader.checksum		= 0;
	ipHeader.sourceIPAddr	= inPacketData->ip->destIPAddr;
	ipHeader.destIPAddr		= inPacketData->ip->sourceIPAddr;
	
	
	u16 temp_checksum 	= 0;
	
	temp_checksum 		= ip_checksum_calc(20, (u8 *)&ipHeader);	
	
	ipHeader.checksum	= ((temp_checksum & 0x00FF) << 8) | ((temp_checksum & 0xFF00) >> 8);

	temp_checksum 		= tcp_checksum_calc(
										20,
										(u8 *)ipHeader.sourceIPAddrOctets,
										(u8 *)ipHeader.destIPAddrOctets,
										0,
										(u8 *)&tcpHeader);
	
	tcpHeader.checksum	= ((temp_checksum & 0x00FF) << 8) | ((temp_checksum & 0xFF00) >> 8);


	Ethernet_Header ethernetHeader;
	
	memcpy(	ethernetHeader.sourceMAC,
			inPacketData->E_Header->destMAC,
			6);
	
	memcpy(	ethernetHeader.destMAC,
			inPacketData->E_Header->sourceMAC,
			6);
	
	ethernetHeader.type = inPacketData->E_Header->type;
	
	memcpy(	&TCPResetRes[dataIndex],
			&ethernetHeader,
			sizeof(ethernetHeader));
	
	dataIndex += sizeof(ethernetHeader);
	
	memcpy(	&TCPResetRes[dataIndex],
			&ipHeader,
			sizeof(ipHeader));
	
	dataIndex += sizeof(ipHeader);
	
	memcpy(	&TCPResetRes[dataIndex],
			&tcpHeader,
			sizeof(tcpHeader));
	
	dataIndex += sizeof(tcpHeader);
	
	overlapped.hEvent = ghEventSend;
	ResetEvent(ghEventSend);

	RnIf_PacketSend(
				ghAdapter,
				TCPResetRes,
				dataIndex,
				&bytesSent,
				&overlapped,
				TRUE);
	
	
	/*
	 * Now send RST from the denied machine to ourself also, to clear
	 * our side of connection
	 */
	dataIndex = 0;

	tcpHeader.sourcePort 	= inPacketData->tcp->sourcePort;
	tcpHeader.destPort 		= inPacketData->tcp->destPort;
	tcpHeader.seqNumber		= inPacketData->tcp->seqNumber;
	tcpHeader.ackNumber		= inPacketData->tcp->ackNumber;
	tcpHeader.hLen			= 0x50;		// we only use 4 high order bits
	tcpHeader.codeBits		= 0x04;		// RST bit set
	tcpHeader.window		= 0;
	tcpHeader.checksum		= 0;
	tcpHeader.urgentPointer	= 0;
	
	ipHeader.verAndHLen		= 0x45;		// Ver = 4, HLEN = 5 (20 bytes)
	ipHeader.tos			= 0;
	ipHeader.totalLength	= 0x2800;	// 20 bytes IP header and 20 TCP
	ipHeader.identification	= ~inPacketData->ip->identification;
	ipHeader.flagsAndOffset	= 0;
	ipHeader.ttl			= 64;
	ipHeader.proto			= TCP_PROTOCOL_NUM;
	ipHeader.checksum		= 0;
	ipHeader.sourceIPAddr	= inPacketData->ip->sourceIPAddr;
	ipHeader.destIPAddr		= inPacketData->ip->destIPAddr;
	
	
	temp_checksum 		= 0;
	
	temp_checksum 		= ip_checksum_calc(20, (u8 *)&ipHeader);
	
	ipHeader.checksum	= ((temp_checksum & 0x00FF) << 8) | ((temp_checksum & 0xFF00) >> 8);
	
	temp_checksum 		= tcp_checksum_calc(
										20,
										(u8 *)ipHeader.sourceIPAddrOctets,
										(u8 *)ipHeader.destIPAddrOctets,
										0,
										(u8 *)&tcpHeader);
	
	tcpHeader.checksum	= ((temp_checksum & 0x00FF) << 8) | ((temp_checksum & 0xFF00) >> 8);



	memcpy(	ethernetHeader.sourceMAC,
			inPacketData->E_Header->sourceMAC,
			6);
	
	memcpy(	ethernetHeader.destMAC,
			inPacketData->E_Header->destMAC,
			6);
	
	ethernetHeader.type = inPacketData->E_Header->type;
	
	memcpy(	&TCPResetRes[dataIndex],
			&ethernetHeader,
			sizeof(ethernetHeader));
	
	dataIndex += sizeof(ethernetHeader);
	
	memcpy(	&TCPResetRes[dataIndex],
			&ipHeader,
			sizeof(ipHeader));
	
	dataIndex += sizeof(ipHeader);
	
	memcpy(	&TCPResetRes[dataIndex],
			&tcpHeader,
			sizeof(tcpHeader));
	
	dataIndex += sizeof(tcpHeader);
	
	overlapped.hEvent = ghEventSend;
	ResetEvent(ghEventSend);

	RnIf_PacketSend(	
				ghAdapter,
				TCPResetRes,
				dataIndex,
				&bytesSent,
				&overlapped,
				TRUE);
}



/****************************************************************************
 *  FUNCTION NAME:  ip_checksum_calc
 *
 *  DESCRIPTION:    calculate the checksum of IP header
 *
 *  NOTES:          
 *
 ***************************************************************************/
u16
ip_checksum_calc(	u16 len_in_octets,
					u8 	*header_buff)
{
	u16 word16;
	u32 sum=0;
	u16 i;
    
	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i = 0; i < len_in_octets; i = i + 2)
	{
		word16 =((header_buff[i] << 8) & 0xFF00) + (header_buff[i+1] & 0xFF);
		sum = sum + (u32) word16;
	}
	
	// take only 16 bits out of the 32 bit sum and add up the carries
	while (sum >> 16)
	{
		sum = (sum & 0xFFFF) + (sum >> 16);
	}

	// one's complement the result
	sum = ~sum;
	
	return ((u16) sum);
}


/****************************************************************************
 *  FUNCTION NAME:  tcp_checksum_calc
 *
 *  DESCRIPTION:    calculate the checksum for tcp
 *
 *  NOTES:          
 *
 ***************************************************************************/
u16
tcp_checksum_calc(	u16		len_tcp, 
					u8 		*src_addr,
					u8 		*dest_addr,
					BOOL 	padding,
					u8 		*buff)
{
	u16 prot_tcp = 6;
	u16 padd = 0;
	u16 word16;
	u32 sum;
	u16 i = 0;
	
	// Find out if the length of data is even or odd number. If odd,
	// add a padding byte = 0 at the end of packet
	if ((padding & 1) == 1)
	{
		padd = 1;
		buff[len_tcp] = 0;
	}
	
	//initialize sum to zero
	sum = 0;
	
	// make 16 bit words out of every two adjacent 8 bit words and 
	// calculate the sum of all 16 vit words
	for (i = 0; i < len_tcp + padd; i = i + 2)
	{
		word16 	= ((buff[i] << 8) & 0xFF00) + (buff[i + 1] & 0xFF);
		sum 	= sum + (u32)word16;
	}	
	
	// add the TCP pseudo header which contains:
	// the IP source and destinationn addresses,
	for (i = 0; i < 4; i = i + 2)
	{
		word16 	= ((src_addr[i] << 8) & 0xFF00) + (src_addr[i + 1] & 0xFF);
		sum		= sum + word16;	
	}
	
	for (i = 0; i < 4; i = i + 2)
	{
		word16 	= ((dest_addr[i] << 8) & 0xFF00) + (dest_addr[i + 1] & 0xFF);
		sum		= sum + word16; 	
	}
	
	// the protocol number and the length of the TCP packet
	sum = sum + prot_tcp + len_tcp;

	// keep only the last 16 bits of the 32 bit calculated sum and add the carries
    while (sum>>16)
	{
		sum = (sum & 0xFFFF) + (sum >> 16);
	}
		
	// Take the one's complement of sum
	sum = ~sum;

	return ((u16) sum);
}

/****************************************************************************
 *  FUNCTION NAME:  ConvertStrIPToNumber
 *
 *  DESCRIPTION:    Converts the given IP address from string format to
 *					integer format. The converted integer is represented in
 *					little-endian form, which means an ip address 
 *					01.dd.3d.fd will be represented as fd3ddd01 in integer
 *
 *  NOTES:          
 *
 ***************************************************************************/
unsigned long
ConvertStrIPToNumber(std::string &inDenyIPStr)
{
	int nFirstDot = 0;
	int nSecondDot = 0;

	int	quad[4] = {0};

	for (int i = 0; i < 4; ++i)
	{
		nSecondDot = inDenyIPStr.find(L'.', nFirstDot);

		/*
		 * Only last quad won't end with a '.'
		 */
		if (nSecondDot == -1)
		{
			if (i != 3)
			{
				/*
				 * Malformed IP address
				 */
				return 0;
			}
			else
			{
				nSecondDot = inDenyIPStr.length();
			}
		}		

		if (nSecondDot > nFirstDot)
		{
			quad[i] = atol(inDenyIPStr.substr(nFirstDot, nSecondDot - nFirstDot).c_str());
			nFirstDot = nSecondDot + 1;
		}
		else
		{
			/*
			 * Malformed IP address
			 */
			return 0;
		}
	}

	return ((quad[3] << 24) | (quad[2] << 16) | (quad[1] << 8) | (quad[0]));
}

/****************************************************************************
 *  FUNCTION NAME:  ActivateSniffer
 *
 *  DESCRIPTION:    Activates the sniffer on given interface
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
ActivateSniffer(RNIF_ADAPTER_INFO& inAdapterInfo)
{
	if (!RnIf_DriverIsRunning())
	{
		goto Cleanup;
	}

	if (!RnIf_AdapterBind(inAdapterInfo))
	{
		goto Cleanup;
	}

	ghAdapter = RnIf_AdapterOpen(inAdapterInfo);

	if (ghAdapter == INVALID_HANDLE_VALUE)
	{
		goto Cleanup;
	}

	ghEventSend	= CreateEvent(NULL, FALSE, FALSE, NULL);

	if (ghEventSend == NULL)
	{
		goto Cleanup;
	}

	ghEventReceive = CreateEvent(NULL, FALSE, FALSE, NULL);

	if (ghEventReceive == NULL)
	{
		goto Cleanup;
	}

	ghEventAdapter = CreateEvent(NULL, FALSE, FALSE, NULL);

	if (ghEventAdapter == NULL)
	{
		goto Cleanup;
	}

	// if adapter is opened
	SetFilterDirected();

	// Cause the thread to run till we set it to false
	gSnifferRun = true;

	// start the send receive thread
	gThreadHandle = CreateThread(
								NULL,
								0,
								SnifferThread,
								gView,
								0,
								&gThreadId);

	return true;

Cleanup:

	if (ghAdapter != INVALID_HANDLE_VALUE)
	{
		CloseHandle(ghAdapter);
		ghAdapter = INVALID_HANDLE_VALUE;
	}

	if (ghEventSend)
	{
		CloseHandle(ghEventSend);
		ghEventSend = NULL;
	}

	if (ghEventReceive)
	{
		CloseHandle(ghEventReceive);
		ghEventReceive = NULL;
	}

	if (ghEventAdapter)
	{
		CloseHandle(ghEventAdapter);
		ghEventAdapter = NULL;
	}

	return false;
}


/****************************************************************************
 *  FUNCTION NAME:  DeactivateSniffer
 *
 *  DESCRIPTION:    Deactivates the sniffer on given interface
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
DeactivateSniffer()
{
	if (RnIf_DriverIsRunning())
	{
		// terminate send/receive threads
		gSnifferRun = false;
		Sleep(2000);
		TerminateThread(gThreadHandle, 0);

		/*
		 * Reset the adapter mode to directed if it is set to promiscuous and
		 * then Close handle to the adapter. Don't leave the adapter in
		 * promiscuous mode or it will substantially reduce the network
		 * performance
		 */
		SetFilterDirected();
		RnIf_AdapterClose(ghAdapter);
		ghAdapter = INVALID_HANDLE_VALUE;
	}

	return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  SetFilterDirected
 *
 *  DESCRIPTION:    Only receive packets which are meant for us
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
SetFilterDirected()
{
	if (RnIf_DriverIsRunning())
	{
		OVERLAPPED overlapped;
		memset(&overlapped, 0, sizeof(OVERLAPPED));
		overlapped.hEvent = ghEventAdapter;
		ResetEvent(ghEventAdapter);
		RnIf_AdapterSetModeDirected(ghAdapter, &overlapped);
	}

	return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  SetFilterPromiscuous
 *
 *  DESCRIPTION:    Receive all the packets which are coming to us
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
SetFilterPromiscuous()
{
	if (RnIf_DriverIsRunning())
	{
		OVERLAPPED overlapped;
		memset(&overlapped, 0, sizeof(OVERLAPPED));
		overlapped.hEvent = ghEventAdapter;
		ResetEvent(ghEventAdapter);
		RnIf_AdapterSetModePromiscuous(ghAdapter, &overlapped);
	}

	return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  ProcessPacket
 *
 *  DESCRIPTION:    Process the newly received packet and apply firewall
 *					rules
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
ProcessPacket(PacketData *pPacketData)
{
	unsigned short frameType = ((pPacketData->E_Header->type & 0x00FF) << 8) |
								((pPacketData->E_Header->type & 0xFF00) >> 8);
	
	if (frameType <= ETHERNET_MAXIMUM_DATA_LENGTH)
	{
		pPacketData->ethernetType	= ETHERNET_802_3;
		
		frameType	= ((pPacketData->E_802_3_Data->type & 0x00FF) << 8) | 
					  ((pPacketData->E_802_3_Data->type & 0xFF00) >> 8);
		
		pPacketData->raw_ip_data	= pPacketData->E_802_3_Data->ip_data;
	}
	else
	{
		pPacketData->ethernetType	= ETHERNET_II;
		pPacketData->raw_ip_data	= pPacketData->E_II_Data->ip_data;
	}

	if (frameType == IPV4_DATAGRAM)
	{
		pPacketData->datagramType = IPV4_DATAGRAM;
		
		/*
		 * In this application we only care for TCP packets in IPV4
		 */		
		if (pPacketData->ip->proto == TCP_PROTOCOL_NUM)
		{
			/*
		  	 * This is a TCP Packet. Get the start of TCP data from the IP data
			 */
			pPacketData->raw_tcp_data = &pPacketData->raw_ip_data[(pPacketData->ip->verAndHLen & 0x0F) * 4];
		}
	}
	else if (frameType == IPV6_DATAGRAM)
	{
		pPacketData->datagramType = IPV6_DATAGRAM;
	}
	else if (frameType == ARP_DATAGRAM)
	{
		pPacketData->datagramType = ARP_DATAGRAM;
	}
	else if (frameType == RARP_DATAGRAM)
	{
		pPacketData->datagramType = RARP_DATAGRAM;
	}
	else
	{
		pPacketData->datagramType = UNKNOWN;
	}

	if (gFirewallActivated)
	{
		ApplyFirewallOnPacket(pPacketData);
	}

	return 0;
}


/* -----------------------------------------------------------------------
 * Function Name:
 * 		SnifferThread
 *
 * Description:
 *		Sniffs the adapter for all the packets coming in
 * ---------------------------------------------------------------------*/
DWORD WINAPI
SnifferThread(LPVOID pParam)
{
	CSfView *pView = (CSfView *)pParam;

	while(gSnifferRun)
	{
		UCHAR receiveBuffer[MAX_ETHERNET_PACKET_LENGTH];
		ULONG packetLength = 0;
		OVERLAPPED overlapped;

		memset(&overlapped, 0, sizeof(OVERLAPPED));
		overlapped.hEvent = ghEventReceive;
		ResetEvent(ghEventReceive);

		// Read packet
		if (RnIf_PacketReceive(	ghAdapter,
								receiveBuffer,
								MAX_ETHERNET_PACKET_LENGTH,
								&packetLength,
								&overlapped,
								TRUE))
		{
			PacketData *pPacketData = new PacketData(
												packetLength,
												receiveBuffer);

			/*
			 * Process the packet to retrieve basic information
			 */
			ProcessPacket(pPacketData);
			pView->HandleNewPacket(pPacketData);
		}
	}

	return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  GetSourceMacAddress
 *
 *  DESCRIPTION:    Returns the source MAC address as a string from given
 *					given ethernet frame.
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
GetSourceMacAddress(std::wstring &oStr, PacketData *pPacket)
{
	RepresentBytesAsHexString(	oStr,
								pPacket->E_Header->sourceMAC,
								0,
								5);
}


/****************************************************************************
 *  FUNCTION NAME:  GetDestMacAddress
 *
 *  DESCRIPTION:    Returns the destination MAC address as a string from
 *					from given ethernet frame.
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
GetDestMacAddress(std::wstring &oStr, PacketData *pPacket)
{
	RepresentBytesAsHexString(	oStr,
								pPacket->E_Header->destMAC,
								0,
								5);
}


/****************************************************************************
 *  FUNCTION NAME:  GetDatagramType
 *
 *  DESCRIPTION:    Returns the datagram type as a string from
 *					from the given ethernet frame.
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
GetDatagramTypeString(std::wstring &oStr, PacketData *pPacket)
{
	if (pPacket->datagramType == IPV4_DATAGRAM)
	{
		oStr = ETHERNET_CONTAINS_IPV4_FRAME_STR;
	}
	else if (pPacket->datagramType == IPV6_DATAGRAM)
	{
		oStr = ETHERNET_CONTAINS_IPV6_FRAME_STR;
	}
	else if (pPacket->datagramType == ARP_DATAGRAM)
	{
		oStr = ETHERNET_CONTAINS_ARP_FRAME_STR;
	}
	else if (pPacket->datagramType == RARP_DATAGRAM)
	{
		oStr = ETHERNET_CONTAINS_RARP_FRAME_STR;
	}	
	else
	{
		oStr = ETHERNET_CONTAINS_UNKNOWN_FRAME_STR;
	}
}


/****************************************************************************
 *  FUNCTION NAME:  GetDatagramType
 *
 *  DESCRIPTION:    Returns true if packet contains IPV4 datagram, otherwise
 *					it returns false
 *
 *  NOTES:          
 *
 ***************************************************************************/
bool
IsIPV4Datagram(PacketData *inPacketData)
{
	return (inPacketData->datagramType == IPV4_DATAGRAM);
}


/****************************************************************************
 *  FUNCTION NAME:  GetEthernetProtocolName
 *
 *  DESCRIPTION:    Returns the ethernet protocol version name like 802.2 or
 *					802.3
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
GetEthernetProtocolName(std::wstring &oStr, PacketData *pPacket)
{
	if (pPacket->ethernetType == ETHERNET_802_3)
	{
		oStr = ETHERNET_FORMAT_802_3_STR;
	}
	else
	{
		oStr = ETHERNET_FORMAT_ETHERNET_II_STR;
	}
}


/****************************************************************************
 *  FUNCTION NAME:  GetSourceIP
 *
 *  DESCRIPTION:    Returns the source IP Address from the given packet
 *
 *  NOTES:          This function assumes that the packet contains IPV4
 *					datagram
 *
 ***************************************************************************/
void
GetSourceIP(std::wstring &oStr, PacketData *pPacket)
{
	ConvertBytesToIPString(oStr, pPacket->ip->sourceIPAddrOctets);
}


/****************************************************************************
 *  FUNCTION NAME:  GetDestIP
 *
 *  DESCRIPTION:    Returns the destination IP Address from the given packet
 *
 *  NOTES:          This function assumes that the packet contains IPV4
 *					datagram
 *
 ***************************************************************************/
void
GetDestIP(std::wstring &oStr, PacketData *pPacket)
{
	ConvertBytesToIPString(oStr, pPacket->ip->destIPAddrOctets);
}


/****************************************************************************
 *  FUNCTION NAME:  ConvertBytesToIPString
 *
 *  DESCRIPTION:    Converts 4 bytes from given address to IP address
 *					notation like 172.32.33.123
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
ConvertBytesToIPString(	std::wstring &oStr,
						unsigned char *inData)
{
	for (int i = 0; i < 4; ++i)
	{
		wchar_t ch[5] = {0};

		if (i == 0)
		{
			swprintf(ch, L"%d\0", inData[i]);
		}
		else
		{
			swprintf(ch, L".%d\0", inData[i]);
		}

		oStr += ch;
	}
}



/****************************************************************************
 *  FUNCTION NAME:  RepresentBytesAsHexString
 *
 *  DESCRIPTION:    Change given bytes to their equivalent Hex representation
 *					and concatenate them together
 *					e.g. for bytes 1, 23, 16 the resulting string will become
 *					"01 17 10 "
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
RepresentBytesAsHexString(	std::wstring &oStr,
							void *inData,
							int inStart,
							int inEnd)
{
	for (int i = inStart; i < inEnd; ++i)
	{
		wchar_t ch[5] = {0};
		swprintf(ch, L"%02X \0", ((unsigned char*)inData)[i]);
		oStr += ch;
	}
}


/****************************************************************************
 *  FUNCTION NAME:  PrepareHexDump
 *
 *  DESCRIPTION:    Makes a multi-line hex string of given number of bytes
 *					Each line contains at max maxBytesInOneLine bytes
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
PrepareHexDump(	std::wstring &oStr,
				void *inData,
				int inStart,
				int inEnd,
				int maxBytesInOneLine)
{
	std::wstring tempStr;

	/*
	 * Generate Hex Dump Header, so that it is easy to analyze
	 */
	for (int i = 0; i < maxBytesInOneLine; ++i)
	{
		wchar_t ch[5] = {0};
		swprintf(ch, L"%02d \0", i);
		tempStr += ch;
	}
	oStr += tempStr;
	oStr += L"\r\n";

	tempStr.erase();
	for (int i = 0; i < maxBytesInOneLine; ++i)
	{
		tempStr += L"== ";
	}
	oStr += tempStr;
	oStr += L"\r\n";

	/*
	 * Generate the actual hex dump
	 */
	int nTotalNumLines = ((inEnd - inStart) / maxBytesInOneLine);

	if (((inEnd - inStart) % maxBytesInOneLine) != 0)
	{
		++nTotalNumLines;
	}

	for (int i = 0; i < nTotalNumLines; ++i)
	{
		tempStr.erase();

		int startPos	= (i * maxBytesInOneLine);
		int endPos		= ((i + 1) * maxBytesInOneLine);

		endPos = (endPos > inEnd) ? inEnd : endPos;

		RepresentBytesAsHexString(	tempStr,
									inData,
									startPos,
									endPos);

		oStr += tempStr;
		oStr += L"\r\n";
	}
}



bool
ActivateFirewall(wchar_t *fileName)
{
	gDenyIPList.clear();

	/*
	 * We open the file here and load the deny list. Once that list is loaded,
	 * we can deny access to listed hosts if they try to connect to us by
	 * replying with TCP RST on their SYN packets
	 */
	FILE *accessFile;

	accessFile = _wfopen(fileName, L"rt");

	if (accessFile)
	{
		char szLine[MAX_PATH];
		
		while (fgets(szLine, MAX_PATH, accessFile))
		{
			if (strlen(szLine) > 0)
			{
				/*
				 * If we read \n also, then remove it
				 */
				if (szLine[strlen(szLine) - 1] == '\n')
				{
					szLine[strlen(szLine) - 1] = 0;
				}

				std::string denyIPStr = szLine;
				if (denyIPStr.length() > 0)
				{
					unsigned long denyIP = ConvertStrIPToNumber(denyIPStr);

					if (denyIP > 0)
					{
						gDenyIPList.push_back(denyIP);
					}
				}
			}
		}

		fclose(accessFile);

		gFirewallActivated = true;
	}
	else
	{
		gFirewallActivated = false;
	}

	return gFirewallActivated;
}


void
DeactivateFirewall()
{
	gDenyIPList.clear();
	gFirewallActivated = false;
}

void
SfInitialize(CSfView *pView)
{
	gView = pView;
}
